@@ -42,7 +42,7 @@ BREW_DEPRECATION_MSG = (
     "brew uninstall cumulusci-plus\nbrew install pipx\npipx ensurepath\npipx install cumulusci-plus"
 )
 PIP_UPDATE_CMD = "pip install --upgrade cumulusci-plus"
-PIPX_UPDATE_CMD = "pipx upgrade cumulusci-plus"
+PIPX_UPDATE_CMD = "pipx install cumulusci-plus-azure-devops --include-deps --force"
 
 
 def parse_api_datetime(value):
@@ -245,6 +245,10 @@ def inject_namespace(
 ):
     """Replaces %%%NAMESPACE%%% in file content and ___NAMESPACE___ in file name
     with either '' if no namespace is provided or 'namespace__' if provided.
+
+    Also handles:
+    - %%%MANAGED_OR_NAMESPACED_ORG%%% and ___MANAGED_OR_NAMESPACED_ORG___ tokens
+      which are replaced with 'namespace__' if managed=True OR namespaced_org=True
     """
 
     # Handle namespace and filename tokens
@@ -264,7 +268,15 @@ def inject_namespace(
     # Handle tokens %%%NAMESPACED_ORG%%% and ___NAMESPACED_ORG___
     namespaced_org_token = "%%%NAMESPACED_ORG%%%"
     namespaced_org_file_token = "___NAMESPACED_ORG___"
-    namespaced_org = namespace + "__" if namespaced_org else ""
+    namespaced_org = (namespace + "__") if namespaced_org and namespace else ""
+
+    # Handle tokens %%%NAMESPACED_ORG_COLON%%%
+    namespaced_org_colon_token = "%%%NAMESPACED_ORG_COLON%%%"
+    namespaced_org_colon = (namespace + ":") if namespaced_org and namespace else ""
+
+    # Handle tokens %%%NAMESPACE_COLON%%%
+    namespace_colon_token = "%%%NAMESPACE_COLON%%%"
+    namespace_colon = (namespace + ":") if managed and namespace else ""
 
     # Handle token %%%NAMESPACE_OR_C%%% for lightning components
     namespace_or_c_token = "%%%NAMESPACE_OR_C%%%"
@@ -274,6 +286,13 @@ def inject_namespace(
     namespaced_org_or_c_token = "%%%NAMESPACED_ORG_OR_C%%%"
     namespaced_org_or_c = namespace if namespaced_org else "c"
 
+    # Handle new tokens %%%MANAGED_OR_NAMESPACED_ORG%%% and ___MANAGED_OR_NAMESPACED_ORG___
+    managed_or_namespaced_org_token = "%%%MANAGED_OR_NAMESPACED_ORG%%%"
+    managed_or_namespaced_org_file_token = "___MANAGED_OR_NAMESPACED_ORG___"
+    managed_or_namespaced_org = (
+        (namespace + "__") if ((managed) or (namespaced_org)) and namespace else ""
+    )
+
     orig_name = name
     prev_content = content
     content = content.replace(namespace_token, namespace_prefix)
@@ -317,6 +336,20 @@ def inject_namespace(
             f'  {name}: Replaced {namespaced_org_token} with "{namespaced_org}"'
         )
 
+    prev_content = content
+    content = content.replace(namespaced_org_colon_token, namespaced_org_colon)
+    if logger and content != prev_content:
+        logger.info(
+            f'  {name}: Replaced {namespaced_org_colon_token} with "{namespaced_org_colon}"'
+        )
+
+    prev_content = content
+    content = content.replace(namespace_colon_token, namespace_colon)
+    if logger and content != prev_content:
+        logger.info(
+            f'  {name}: Replaced {namespace_colon_token} with "{namespace_colon}"'
+        )
+
     prev_content = content
     content = content.replace(namespaced_org_or_c_token, namespaced_org_or_c)
     if logger and content != prev_content:
@@ -324,9 +357,20 @@ def inject_namespace(
             f'  {name}: Replaced {namespaced_org_or_c_token} with "{namespaced_org_or_c}"'
         )
 
+    # Replace new managed_or_namespaced_org token in content
+    prev_content = content
+    content = content.replace(
+        managed_or_namespaced_org_token, managed_or_namespaced_org
+    )
+    if logger and content != prev_content:
+        logger.info(
+            f'  {name}: Replaced {managed_or_namespaced_org_token} with "{managed_or_namespaced_org}"'
+        )
+
     # Replace namespace token in file name
     name = name.replace(filename_token, namespace_prefix)
     name = name.replace(namespaced_org_file_token, namespaced_org)
+    name = name.replace(managed_or_namespaced_org_file_token, managed_or_namespaced_org)
     if logger and name != orig_name:
         logger.info(f"  {orig_name}: renamed to {name}")
 
@@ -737,11 +781,12 @@ def get_tasks_with_options(project_config, frozen_options: frozenset):
     coordinator_opts = {}
     options = dict(frozen_options)
     for task_config in project_config.list_tasks():
-        if any(
-            key in options.keys()
-            for key in project_config.get_task(task_config["name"])
-            .get_class()
-            .task_options.keys()
-        ):
-            coordinator_opts[task_config["name"]] = options
+        task_keys = (
+            project_config.get_task(task_config["name"]).get_class().task_options.keys()
+        )
+        if any(key in options.keys() for key in task_keys):
+            # only assign options that are actually used by the task. This is to avoid passing options that are not used by the task.
+            coordinator_opts[task_config["name"]] = {
+                key: options[key] for key in options.keys() if key in task_keys
+            }
     return coordinator_opts
